/*
 * Copyright (c) 2005 TopCoder, Inc., All Rights Reserved
 */
package com.topcoder.network.synchronization.defaultimpl;

import java.util.List;

import com.topcoder.network.synchronization.bean.Message;
import com.topcoder.network.synchronization.bean.PeerGroupInfo;
import com.topcoder.network.synchronization.bean.PeerInfo;
import com.topcoder.network.synchronization.exception.DistributedProtocolException;
import com.topcoder.network.synchronization.protocol.DistributedSynchronizationProtocol;
import com.topcoder.network.synchronization.protocol.ErrorHandlingListener;
import com.topcoder.network.synchronization.protocol.MessageListener;
import com.topcoder.network.synchronization.protocol.PeerGroupService;

/**
 * It is the default implementation of the DistributedSynchronizedProtocol, it uses UDP as under-side data transfer protocol,
 * and privades two services: global service and synchronized peer group service.
 * The global service lets the calling application lookup all the peer groups in the subnet.
 * 
 * And the synchronized peer group service is used to send messages to the peers in the group, and each of them 
 * will process the messages in the same order. The whole process of a messsage can be described as following:
 * 1. One peer send the message(A) to the peer group.
 * 
 * 2. In each peer of the group who received A, A will be processed if and only if equal or more than keepSynchronizedPerecent * (number of peers in group)
 * 	  peers received A, and an error message containing which peers didn't receive A will be broadcast in the group. 
 * 	  
 * 	  2.1 Peers didn't receive A will be marked as un-synchronized peer in the peers received A.
 * 	  2.2 Un-synchronized peer can call reSynchronize method to re-synchronize with the peers in the group.
 * 
 * 3. For each peers processing A, if the process is successful, a successful message is bloadcasted, otherwise, an error message is bloadcasted.
 * 	  The calling application can deal with the error message before the next message (B) processed.
 * 
 * 4. The phase to deal a message in a peer group is ended. It's the time to deal the next message.      	
 * 
 * The detail algorithm of the process is in CS and  SynchronizedPeerGroupService's doc.
 * 
 * This class isn't thread-safe. One instance of the protocol is for one thread and one peer group.
 * 
 * @author TCSDESIGNER
 * @version 1.0
 */
public class DistributedSynchronizationProtocolImpl implements DistributedSynchronizationProtocol {
    
    private static final String protocolId = "DEFAULT_DISTRIBUTED_PROTOCOL";
    
    private PeerInfoImpl peerInfo = null;
    
    private static PeerGroupService globalService = null;
    
    private PeerGroupService currentGroupService = null;
    
    public DistributedSynchronizationProtocolImpl() {
        
    }
    
    public DistributedSynchronizationProtocolImpl(PeerInfo peerInfo) {
        
    }

    public String getProtocolId() {
        return null;
    }

    
    public void start() throws DistributedProtocolException {
        
    }

    public void stop() throws DistributedProtocolException {
        this.leavePeerGroup();
        globalService.stop();
    }

    public List getAllPeerGroups(int timeout) throws DistributedProtocolException {
        /*
        ((GlobalMessageListener) this.globalService.getListener()).clear();
        MessageHeader header = new MessageHeader(this.peerInfo.getPeerInfoId(), this.peerInfo.getCredential());
        Message message = new Message(IDGeneratorUtil.getNextId(), header, Message.LOOKUPGROUP_MESSAGE);
        this.globalService.send(message);
        
        try {
            Thread.sleep(timeout);
        } catch (InterruptedException e) {
            
        }
        
        return ((GlobalMessageListener) this.globalService.getListener()).getGroups();
        */
        return null;
    }

    public void joinPeerGroup(PeerGroupInfo groupInfo) throws DistributedProtocolException {


    }

    public void leavePeerGroup() throws DistributedProtocolException {


    }
    
    /* (non-Javadoc)
     * @see com.topcoder.network.synchronization.protocol.DistributedSynchronizationProtocol#MoveToPeerGroup(com.topcoder.network.synchronization.bean.PeerGroupInfo)
     */
    public void MoveToPeerGroup(PeerGroupInfo groupInfo) throws DistributedProtocolException {
        // TODO Auto-generated method stub

    }

    
    public void sentMessage(Message message) throws DistributedProtocolException {

        
    }

    /* (non-Javadoc)
     * @see com.topcoder.network.synchronization.protocol.DistributedSynchronizationProtocol#addApplicationMessageListener(com.topcoder.network.synchronization.protocol.MessageListener)
     */
    public void addApplicationMessageListener(MessageListener listener) {
        // TODO Auto-generated method stub

    }
    /* (non-Javadoc)
     * @see com.topcoder.network.synchronization.protocol.DistributedSynchronizationProtocol#getApplicationMessageListener()
     */
    public MessageListener getApplicationMessageListener() {
        // TODO Auto-generated method stub
        return null;
    }
    /* (non-Javadoc)
     * @see com.topcoder.network.synchronization.protocol.DistributedSynchronizationProtocol#removeApplicationMessageListener(com.topcoder.network.synchronization.protocol.MessageListener)
     */
    public void removeApplicationMessageListener(MessageListener listener) {
        // TODO Auto-generated method stub

    }
    
    public void setErrorHandlingListener(ErrorHandlingListener listener) {
        // TODO Auto-generated method stub

    }

    public ErrorHandlingListener getErrorHandlingListener() {
        // TODO Auto-generated method stub
        return null;
    }

    
        
    /* (non-Javadoc)
     * @see com.topcoder.network.synchronization.protocol.DistributedSynchronizationProtocol#getPeerGroupInfo()
     */
    public PeerGroupInfo getPeerGroupInfo() {
        // TODO Auto-generated method stub
        return null;
    }
    
    /* (non-Javadoc)
     * @see com.topcoder.network.synchronization.protocol.DistributedSynchronizationProtocol#getPeerInfo()
     */
    public PeerInfo getPeerInfo() {
        // TODO Auto-generated method stub
        return null;
    }
    
    public void setPeerInfo(PeerInfoImpl peerInfo) {
        
    }
    
    public void setPeerGroupInfo(PeerGroupInfoImpl peerGroupInfo) {
        
    }
    
    public PeerGroupService getPeerGroupService() {
        return null;
    }
    
    public void setServiceState(ServiceState state) {
        
    }
    
    public ServiceState getServiceState() {
        return null;
    }
}
