/*
 * Copyright (c) 2005 TopCoder, Inc., All Rights Reserved
 */
package com.topcoder.network.synchronization.protocol;

import com.topcoder.network.synchronization.bean.Message;
import com.topcoder.network.synchronization.bean.PeerGroupInfo;
import com.topcoder.network.synchronization.exception.DistributedProtocolException;

/**
 * Send messages to the peer group.
 * All the peers in the peer group can receive the messages. 
 * 
 * @author TCSDESIGNER
 * @version 1.0
 */
public interface MessageSender {
    
    /**
     * Set the PeerGroupInfo of this sender.
     * the PeerGroupInfo is used to tell the sender where the message should be sended.
     * 
     * @param groupInfo the PeerGroupInfo of this sender  
     */
    void setPeerGroupInfo(PeerGroupInfo groupInfo);
    
    /**
     * Return the PeerGroupInfo of this sender.
     * 
     * @return the PeerGroupInfo of this sender.
     */
    PeerGroupInfo getPeerGroupInfo();    
    
    /**
     * Set the timeout of this sender. 
     * If one message can't be sended in timeout millseconds, the message is failed to send.
     * 
     * @param timeout the timeout of this sender.
     */
    void setTimeout(int timeout);
    
    /**
     * Return the timeout of this sender. 
     * If one message can't be sended in timeout millseconds, the message is failed to send.
     * 
     * @return the timeout of this sender.
     */
    int getTimeout();
    
    /**
     * Send the message to the peer group.
     * Usually, each peer of this peer group will receive this message.
     * 
     * @param message the message to send
     * @throws DistributedProtocolException if fails to send the message
     */
    void send(Message message) throws DistributedProtocolException; 
}
